# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import logging
import sys

from beradio.mqtt import BERadioMQTTAdapter

"""
Read data from MQTT topic and print on STDOUT.

Synopsis::

  # subscribe to "hiveeyes/#" topic
  python beradio/subscribe.py 192.168.59.103

  # subscribe to "hiveeyes/999/#" topic
  python beradio/subscribe.py 192.168.59.103 999

"""

logger = logging.getLogger(__name__)


class MQTTSubscriber(object):
    def __init__(self, mqtt_broker, mqtt_topic=None):
        self.mqtt_broker = mqtt_broker
        self.mqtt_topic = mqtt_topic

    def setup(self):
        try:
            logger.info('Connecting to MQTT broker "{}"'.format(self.mqtt_broker))
            self.mqtt = BERadioMQTTAdapter(self.mqtt_broker, topic=self.mqtt_topic)
        except:  # noqa:E722
            logger.error('Failed to connect to MQTT broker "{}"'.format(self.mqtt_broker))
            raise

        return self

    def subscribe(self, topic):
        if topic:
            topic += "/#"
        else:
            topic = "#"
        self.mqtt.subscribe(topic)
        while self.mqtt.mqttc.loop() == 0:
            pass

    def __del__(self):
        if hasattr(self, "mqtt"):
            logger.info("Disconnecting from MQTT broker")
            self.mqtt.close()


if __name__ == "__main__":
    subtopic = ""
    if len(sys.argv) >= 3:
        subtopic = sys.argv[2]
    MQTTSubscriber(mqtt_broker=sys.argv[1]).setup().subscribe(subtopic)



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import logging
import sys

import serial

from beradio.decoder import jobee_decode
from beradio.mqtt import BERadioMQTTAdapter
from beradio.network import protocol_factory
from beradio.protocol import BencodeError
from beradio.util import last_error_and_traceback

"""
Read data in Bencode format from serial port, decode and publish via MQTT.

Based on code from
- Andy Piper http://andypiper.co.uk [2011/09/15]
- Didier Donsez [2014‎/11/22]
  http://air.imag.fr/index.php/Mosquitto#Publication_en_Python

Synopsis::

  python serial_to_mqtt.py /dev/ttyUSB0 localhost
  python serial_to_mqtt.py /dev/ttyACM0 localhost

"""

logger = logging.getLogger(__name__)


class SerialToMQTT(object):
    def __init__(self, serial_device, mqtt_broker, mqtt_topic=None, protocol=2):
        self.serial_device = serial_device
        self.mqtt_broker = mqtt_broker
        self.mqtt_topic = mqtt_topic
        self.protocol_class = protocol_factory(protocol)

    def setup(self):

        try:
            # connect to serial port
            logger.info('Connecting to serial port device "{}"'.format(self.serial_device))
            # self.serial = serial.Serial(serial_device, 9600, timeout=20)
            self.serial = serial.Serial(self.serial_device, 115200)
        except:  # noqa:E722
            logger.error('Failed to connect to serial port device "{}"'.format(self.serial_device))
            raise

        try:
            logger.info('Connecting to MQTT broker "{}"'.format(self.mqtt_broker))
            self.mqtt = BERadioMQTTAdapter(self.mqtt_broker, topic=self.mqtt_topic, enable_heartbeat=True)
        except:  # noqa:E722
            logger.error('Failed to connect to MQTT broker "{}"'.format(self.mqtt_broker))
            raise

        return self

    def forward(self):

        try:
            self.serial.flushInput()

            # remain connected to broker
            # read data from serial and publish
            # while self.mqtt.mqttc.loop() == 0:

            # use mqtt.loop_forever() in a separate thread, this will handle
            # automatic reconnecting if connection to broker got lost
            self.mqtt.mqttc.loop_start()

            while True:

                # logger.debug('-' * 42)

                # read line from serial port
                # li999ei99ei1ei2218ei2318ei2462ei2250ee\0\n
                # li999ei99ei1ei2231ei2325ei2443ei2262ee\0\n
                line = self.serial.readline()
                line = line.strip("\r\n\0 ")

                # debug: output line to stdout
                logger.debug('UART data: "{}"'.format(line))

                if line.startswith("#"):
                    continue

                # Decode from CSV format
                if "," in line:
                    line = line.strip()
                    record = line.split(",")

                    # print 'record:'; pprint(record)

                    # Pop node id from first list element
                    nodeid = record.pop(0)

                    # Ignore empty datetime field
                    _ = record.pop(0)

                    # The other elements are the data values
                    values = record

                    data = None

                    # Weight,Outside Temp,Outside Humid,Inside Temp,Inside Humid,H1 Temp,H2 Temp,H3 Temp,H4 Temp,H5 Temp,Voltage  # noqa:E501
                    if nodeid == "2":
                        keys = [
                            "weight",
                            "temp-outside",
                            "humidity-outside",
                            "temp-inside",
                            "humidity-inside",
                            "temp-h1",
                            "temp-h2",
                            "temp-h3",
                            "temp-h4",
                            "temp-h5",
                            "voltage",
                        ]
                        data = dict(zip(keys, values))

                    # Weight,Outside Temp,Outside Humid,Inside Temp,Inside Humid,Brood Temp,Voltage
                    elif nodeid == "3":
                        keys = [
                            "weight",
                            "temp-outside",
                            "humidity-outside",
                            "temp-inside",
                            "humidity-inside",
                            "temp-brood",
                            "voltage",
                        ]
                        data = dict(zip(keys, values))

                    else:
                        logger.error("Could not decode CSV data, unknown nodeid: {}".format(nodeid))

                    if data:
                        # print 'data:'; pprint(data)

                        for key, value in data.items():
                            data[key] = float(data[key])

                        # Build an appropriate message from CSV data
                        message = {
                            "meta": {
                                "network": self.protocol_class.network_id,
                                "gateway": self.protocol_class.gateway_id,
                                "node": nodeid,
                            },
                            "data": data,
                        }
                        self.mqtt.publish_flexible(message)

                    continue

                # Decode JobeeMonitor line format
                # id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;
                if line.startswith("id=") and ";" in line:
                    data = jobee_decode(line)
                    nodeid = data["id"]
                    del data["id"]

                    # Build an appropriate message from Jobee data
                    message = {
                        "meta": {
                            "network": self.protocol_class.network_id,
                            "gateway": self.protocol_class.gateway_id,
                            "node": nodeid,
                        },
                        "data": data,
                    }
                    self.mqtt.publish_flexible(message)

                    continue

                # Decode from Bencode format
                try:
                    data = self.protocol_class.decode(line)
                except BencodeError:
                    continue

                # print 'data:', data

                # publish to MQTT
                if data:
                    raw_sanitized = self.protocol_class.sanitize(line)

                    if self.protocol_class.VERSION == 1:
                        data = self.protocol_class.to_v2(data)

                    self.mqtt.publish_flexible(data, bencode_raw=raw_sanitized)

                # time.sleep(1)

            self.mqtt.mqttc.loop_stop()

        # Handle list index error (i.e. assume no data received)
        except IndexError:
            logger.error("No data received within serial timeout period")
            # self.cleanup()

        # Handle interactive shutdown
        except KeyboardInterrupt:
            logger.info("KeyboardInterrupt received, terminating")
            self.cleanup()

        except Exception as ex:
            logger.error("Unknown exception: {}\n{}".format(ex, last_error_and_traceback()))

    # called on exit
    # close serial, disconnect MQTT
    def cleanup(self):
        logger.info("Shutting down")
        self.serial.close()
        self.mqtt.close()


if __name__ == "__main__":
    SerialToMQTT(serial_device=sys.argv[1], mqtt_broker=sys.argv[2]).setup().forward()



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import json
import logging
import threading
import time
from collections import OrderedDict
from pprint import pformat

from beradio.protocol import BERadioProtocol2

logger = logging.getLogger(__name__)


class BERadioMessage(object):

    protocol_factory = BERadioProtocol2

    def __init__(self, nodeid, profile="h1"):
        """
        >>> message = BERadioMessage(999)

        >>> bytes(message)
        b'd1:#i999e1:_2:h1e'

        >>> message.temperature(21.63, 19.25, 10.92, 13.54)
        >>> bytes(message)
        b'd1:#i999e1:_2:h11:tli2163ei1925ei1092ei1354eee'

        >>> message.humidity(488.0, 572.0)
        >>> bytes(message)
        b'd1:#i999e1:_2:h11:hli48800ei57200ee1:tli2163ei1925ei1092ei1354eee'

        >>> message.weight(106.77)
        >>> bytes(message)
        b'd1:#i999e1:_2:h11:hli48800ei57200ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee'

        """
        self.nodeid = int(nodeid)
        self.profile = str(profile)
        self.payload = {}
        self.protocol = self.protocol_factory()

    def temperature(self, *args):
        self.payload["t"] = self.protocol.encode_values("t", args)

    def humidity(self, *args):
        self.payload["h"] = self.protocol.encode_values("h", args)

    def weight(self, *args):
        self.payload["w"] = self.protocol.encode_values("w", args)

    def clear(self):
        self.payload = {}

    def encode(self):
        data = {
            "#": self.nodeid,
            "_": self.profile,
        }
        data.update(self.payload)
        message = self.protocol.encode_ether(data)
        return message

    def __bytes__(self):
        return self.encode()

    def __repr__(self):
        data = {
            "#": self.nodeid,
            "_": self.profile,
        }
        data.update(self.payload)
        return pformat(data)

    @classmethod
    def decode(cls, payload):
        message = cls.protocol_factory().decode(payload)
        message["data"] = OrderedDict(message["data"])
        return message

    @classmethod
    def json(cls, payload):
        message = cls.protocol_factory().decode(payload)
        return json.dumps(message, sort_keys=True, indent=4)


class ResettableTimer:
    def __init__(self, interval, function):
        self.interval = interval
        self.function = function
        self.make_timer()

    def make_timer(self):
        self.timer = threading.Timer(self.interval, self.function)
        self.timer.start()

    def reset(self):

        if self.timer is None:
            self.make_timer()
        else:
            if not self.timer.finished.is_set():
                self.timer.cancel()

            self.make_timer()


class BERadioMessageDecoder(object):

    REASSEMBLY_TIMEOUT = 2.5

    # Collector mode: Wait until hitting REASSEMBLY_TIMEOUT, then return complete reassembled messages
    MODE_COLLECT = 1

    # Emitter mode: Emit reassembled fragments to all subscribers after hitting REASSEMBLY_TIMEOUT
    MODE_EMIT = 2

    def __init__(self, reassembly_timeout=None):
        self.reassembly_timeout = reassembly_timeout or self.REASSEMBLY_TIMEOUT
        self.mode = self.MODE_COLLECT
        self.timer = ResettableTimer(self.reassembly_timeout, self.release)
        self.ready = threading.Event()
        self.subscribers = []
        self.messages = []

    def subscribe(self, subscriber):
        self.mode = self.MODE_EMIT
        self.subscribers.append(subscriber)

    def release(self):
        logger.info("BERadioMessageDecoder.release")

        # Signal readiness
        self.ready.set()
        time.sleep(0.1)

        if self.mode == self.MODE_EMIT:
            self.emit()

    def emit(self):
        logger.info("BERadioMessageDecoder.emit")

        # Inform all subscribers
        payload = self.assemble()
        for subscriber in self.subscribers:
            subscriber(payload)

        # Clear buffer for next iteration
        self.messages = []

    def read(self, payload):
        logger.info("BERadioMessageDecoder.read")

        # Clear readiness indicator
        self.ready.clear()

        # Reset the timer
        self.timer.reset()

        # Collect the payload fragment
        self.messages.append(payload)

    def wait(self):

        if self.mode == self.MODE_EMIT:
            raise NotImplementedError(
                "The message decoder does not support waiting for complete packets when running in emitter mode"
            )

        logger.info("BERadioMessageDecoder.wait")

        # Wait for reassembly timeout
        self.ready.wait()

        return self.assemble()

    def to_json(self, strip_time=False):
        data = self.wait()
        payload = json.dumps(data, sort_keys=True, indent=4)
        return payload

    def assemble(self):

        if not self.messages:
            return

        data = OrderedDict()
        for payload in self.messages:

            message = BERadioMessage.decode(payload)
            # message['payload'] = payload

            node_id = message["meta"]["node"]

            if node_id not in data:
                data[node_id] = message
                data[node_id]["messages"] = [payload]
            else:
                data[node_id]["meta"].update(message["meta"])
                data[node_id]["data"].update(message["data"])
                data[node_id]["messages"].append(payload)

        return data


if __name__ == "__main__":

    logging.basicConfig(level=logging.INFO)

    parts = []
    parts.append("d1:#i2e1:_2:h11:tli2168ei1393ei3356ei1468ei1700ee1:hlee")
    parts.append("d1:#i2e1:_2:h12:h0li8370ee1:wli53503600ei2590ee1:lli15100eee")
    parts.append("d1:#i3e1:_2:h11:tli2168ee2:h0li930ee1:wli4242eee")
    parts.append("d1:#i2e1:_2:h11:rli-6600eee")

    def emitter(data):
        logger.info("Emit:\n{}".format(json.dumps(data, indent=4)))

    decoder = BERadioMessageDecoder()
    # decoder.subscribe(emitter)

    decoder.read(parts[0])
    time.sleep(1.5)

    decoder.read(parts[1])
    time.sleep(1.5)

    decoder.read(parts[2])
    decoder.read(parts[3])

    data = decoder.wait()
    logger.info("Data:\n{}".format(json.dumps(data, indent=4)))



#!/usr/bin/env python
# https://github.com/greghaskins/gibberish
import random
import string

__all__ = ("generate_word", "generate_words")

initial_consonants = list(
    set(string.ascii_lowercase) - set("aeiou")
    # remove those easily confused with others
    - set("qxc")
    # add some crunchy clusters
    | set(
        [
            "bl",
            "br",
            "cl",
            "cr",
            "dr",
            "fl",
            "fr",
            "gl",
            "gr",
            "pl",
            "pr",
            "sk",
            "sl",
            "sm",
            "sn",
            "sp",
            "st",
            "str",
            "sw",
            "tr",
            "ch",
            "sh",
        ]
    )
)

final_consonants = list(
    set(string.ascii_lowercase) - set("aeiou")
    # remove the confusables
    - set("qxcsj")
    # crunchy clusters
    | set(["ct", "ft", "mp", "nd", "ng", "nk", "nt", "pt", "sk", "sp", "ss", "st", "ch", "sh"])
)

vowels = "aeiou"


def generate_word():
    """Returns a random consonant-vowel-consonant pseudo-word."""
    return "".join(random.choice(s) for s in (initial_consonants, vowels, final_consonants))


def generate_words(wordcount):
    """Returns a list of ``wordcount`` pseudo-words."""
    return [generate_word() for _ in range(wordcount)]


def console_main():
    import sys

    try:
        wordcount = int(sys.argv[1])
    except (IndexError, ValueError):
        wordcount = 1
    print(" ".join(generate_words(wordcount)))


if __name__ == "__main__":
    console_main()



"""
Load tests from doctests.

This plugin implements :func:`handleFile` to load doctests from text files
and python modules.

To disable loading doctests from text files, configure an empty extensions list:

.. code-block :: ini

  [doctest]
  extensions =

"""
import doctest
import os

from nose2 import util
from nose2.events import Plugin

__unittest = True


class DocTestLoader(Plugin):
    configSection = "doctest"
    commandLineSwitch = (None, "with-doctest", "Load doctests from text files and modules")

    def __init__(self):
        self.extensions = self.config.as_list("extensions", [".txt", ".rst"])

    def handleFile(self, event):
        """Load doctests from text files and modules"""
        path = event.path
        _root, ext = os.path.splitext(path)
        if ext in self.extensions:
            optionflags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE | doctest.REPORT_UDIFF
            # +REPORT_ONLY_FIRST_FAILURE
            suite = doctest.DocFileTest(path, module_relative=False, optionflags=optionflags)
            event.extraTests.append(suite)
            return
        elif not util.valid_module_name(os.path.basename(path)):
            return

        name, package_path = util.name_from_path(path)
        util.ensure_importable(package_path)
        try:
            module = util.module_from_name(name)
        except Exception:
            # XXX log warning here?
            return
        if hasattr(module, "__test__") and not module.__test__:
            return
        try:
            suite = doctest.DocTestSuite(module)
        except ValueError:
            # with python <= 3.5, doctest, very annoyingly, raises ValueError
            # when a module has no tests.
            return
        event.extraTests.append(suite)



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015 Andreas Motl <andreas@hiveeyes.org>
"""BERadio spec and reference implementation"""
__appname__ = "beradio"
__version__ = "0.13.0"


def program_name(with_version=False):
    name = __appname__
    if with_version:
        name += " " + __version__
    return name



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015 Andreas Motl <andreas@hiveeyes.org>
import json
import logging
import sys

import bencode
from docopt import DocoptExit, docopt

from beradio import program_name
from beradio.config import BERadioConfig
from beradio.forward import SerialToMQTT
from beradio.network import GatewayIdentifier, NetworkIdentifier, protocol_factory
from beradio.protocol import BERadioProtocolBase
from beradio.publish import DataToMQTT
from beradio.subscribe import MQTTSubscriber
from beradio.util import setup_logging

APP_NAME = program_name(with_version=True)

logger = logging.getLogger(__name__)


def beradio_cmd():
    """
    Usage:
      beradio forward --source=serial:///dev/ttyUSB0 --target=mqtt://localhost [--protocol=<version>] [--log=<log>] [--debug]  # noqa:E501
      beradio decode <payload> [--protocol=<version>] [--debug]
      beradio info
      beradio --version
      beradio (-h | --help)

    Options:
      --source=<source>         Data source, e.g. serial:///dev/ttyUSB0
      --target=<target>         Data sink, e.g. mqtt://localhost
      --protocol=<version>      Protocol version: 1 or 2        [default: 2]
      --log=<log>               Where to send log output        [default: -]
      --version                 Show version information
      --debug                   Enable debug messages
      -h --help                 Show this screen

    """
    options = docopt(beradio_cmd.__doc__, version=APP_NAME)
    # print('options: {}'.format(options))

    source = options.get("--source")
    target = options.get("--target")
    protocol = options.get("--protocol")

    boot_logging(options)

    if options.get("forward"):
        if source.startswith("serial://") and target.startswith("mqtt://"):
            source = source.replace("serial://", "")
            SerialToMQTT(serial_device=source, mqtt_broker=target, protocol=protocol).setup().forward()

        elif source.startswith("data://") and target.startswith("mqtt://"):
            data = source.replace("data://", "")
            if data == "stdin":
                data = sys.stdin.read().strip()

            DataToMQTT(mqtt_broker=target, protocol=protocol).setup().publish(data)

        else:
            raise DocoptExit(
                "Unable to handle combination of {} and {} in forwarding mode".format(
                    options.get("--source"), options.get("--target")
                )
            )

    elif options.get("decode"):
        p = protocol_factory(protocol)
        payload = options.get("<payload>")
        return json.dumps(p.decode_safe(payload), indent=4)

    elif options.get("info"):
        network_id = str(NetworkIdentifier())
        gateway_id = str(GatewayIdentifier())

        print("-" * 50, file=sys.stderr)
        print(APP_NAME.center(50), file=sys.stderr)
        print("-" * 50, file=sys.stderr)
        print("config file: {}".format(BERadioConfig().config_file), file=sys.stderr)
        print("network_id:  {}".format(network_id), file=sys.stderr)
        print("gateway_id:  {}".format(gateway_id), file=sys.stderr)


def bdecode_cmd():
    """
    Synopsis::

        bdecode li999ei99ei1ei2218ei2318ei2462ei2250ee
        [999, 99, 1, 2218, 2318, 2462, 2250]

        bdecode d1:tli3455ei3455ei3455ei3455ee1:hli890ei377ee1:wi12333ee
        {'h': [890, 377], 't': [3455, 3455, 3455, 3455], 'w': 12333}

    """

    boot_logging()

    if len(sys.argv) == 2:
        payload = sys.argv[1].strip()
    else:
        logger.error("bdecode requires Bencode data as single argument")
        sys.exit(1)

    # decode from Bencode format
    try:
        return BERadioProtocolBase().decode_ether(payload)

    except ValueError:
        sys.exit(1)


def bencode_cmd():
    """
    Synopsis::

        beencode [999, 99, 1, 2218, 2318, 2462, 2250]
        li999ei99ei1ei2218ei2318ei2462ei2250ee

        beencode {'h': [890, 377], 't': [3455, 3455, 3455, 3455], 'w': 12333}
        d1:tli3455ei3455ei3455ei3455ee1:hli890ei377ee1:wi12333ee

    """

    boot_logging()

    if len(sys.argv) >= 2:
        return bencode.bencode(sys.argv[1])

    # data_01 = [999, 99, 1, 2218, 2318, 2462, 2250]
    # data_02 = {"t": [1234, 5678, 4242], "h": [587, 476], "w": 42}
    data_03 = {"#": 999, "_": "h1", "t": [2163, 1925, 1092, 1354], "h": [488, 572], "w": 10677}
    return bencode.bencode(data_03)


def bemqtt_cmd():
    """
    Usage:
      bemqtt subscribe --source=mqtt://localhost/this-topic [--topic=<topic>] [--debug]
      bemqtt --version
      bemqtt (-h | --help)

    Options:
      --source=<source>         Data source, e.g. mqtt://localhost
      --topic=<topic>           Additional fragment to append to topic from MQTT URL
      --version                 Show version information
      --debug                   Enable debug messages
      -h --help                 Show this screen

    """

    options = docopt(bemqtt_cmd.__doc__, version=APP_NAME)
    # print('options: {}'.format(options))

    boot_logging(options)

    source = options.get("--source")
    if options.get("subscribe"):
        topic = options.get("--topic")

        if source.startswith("mqtt://"):
            MQTTSubscriber(mqtt_broker=source).setup().subscribe(topic)

        else:
            raise NotImplementedError("Unable to subscribe to data source {}".format(source))


def boot_logging(options=None):
    options = options or {}
    log_level = logging.INFO
    output = "-"
    if options.get("--log"):
        output = options["--log"]
    if options.get("--debug"):
        log_level = logging.DEBUG

    setup_logging(output=output, level=log_level)



# -*- coding: utf-8 -*-
# (c) 2014-2022 Andreas Motl <andreas@hiveeyes.org>
import logging
import os
import socket
import sys
import traceback
from calendar import timegm
from datetime import datetime
from io import StringIO
from math import sin
from uuid import uuid4

import json_store
from appdirs import user_data_dir

from beradio.gibberish import generate_word


class Singleton(object):
    """
    Singleton class by Duncan Booth.
    Multiple object variables refers to the same object.
    http://www.suttoncourtenay.org.uk/duncan/accu/pythonpatterns.html
    """

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Singleton, cls).__new__(cls)
        return cls._instance


class ConfigStoreJson(dict):

    store = None
    appname = "DEFAULT"

    def __init__(self):
        self.app_data_dir = user_data_dir(self.appname)
        # print("ConfigStoreJson app_data_dir:", self.app_data_dir, file=sys.stderr)
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir)
        self.config_file = os.path.join(self.app_data_dir, "config.json")
        # print("ConfigStoreJson config_file:", self.config_file, file=sys.stderr)

        self.setup()

    def setup(self):
        if not ConfigStoreJson.store:
            # print("ConfigStoreJson.__init__", file=sys.stderr)
            ConfigStoreJson.store = json_store.open(self.config_file)

    def has_key(self, key):
        return key in ConfigStoreJson.store

    def __getitem__(self, key):
        # print('ConfigStoreJson.__getitem__', file=sys.stderr)
        return ConfigStoreJson.store[key]

    def __setitem__(self, key, value):
        # print('ConfigStoreJson.__setitem__', key, value, file=sys.stderr)
        ConfigStoreJson.store[key] = value
        ConfigStoreJson.store.sync()


class UUIDGenerator(object):
    def get_next_id(self):
        return uuid4()


class PersistentUniqueIdentifier(Singleton):

    config = None
    identifier = "UNKNOWN"
    attribute = "uuid"
    store_class = ConfigStoreJson
    id_generator = UUIDGenerator().get_next_id

    def __init__(self):
        if not self.config:
            self.config = self.store_class()
        if self.attribute not in self.config or self.config[self.attribute] is None:
            self.config[self.attribute] = str(self.id_generator())
        self.identifier = self.config[self.attribute]
        # print(self.attribute + ':', self.identifier, file=sys.stderr)

    def __str__(self):
        return str(self.identifier)


def human_unique_id(*args):
    """
    Produces random, pronounceable pseudo-words combined with the current second.
    Examples: swan4, zech55, tug22, drew1

    .. seealso::

        https://github.com/greghaskins/gibberish
        http://www.anotherchris.net/csharp/friendly-unique-id-generation-part-2/#time
    """
    now = datetime.now()
    word = generate_word()
    unique = str(word) + str(now.second)
    return unique


def setup_logging(output="-", level=logging.INFO):
    log_format = "%(asctime)-15s [%(name)-20s] %(levelname)-7s: %(message)s"

    if output == "-":
        logging.basicConfig(stream=sys.stderr, format=log_format, level=level)

    else:
        logging.basicConfig(filename=output, format=log_format, level=level)


def timestamp_nanos():
    """
    https://docs.influxdata.com/influxdb/v0.13/troubleshooting/frequently_encountered_issues/#querying-outside-the-min-max-time-range

        Smallest valid timestamp: -9023372036854775808 (approximately 1684-01-22T14:50:02Z)
        Largest valid timestamp: 9023372036854775807 (approximately 2255-12-09T23:13:56Z)
    """
    timestamp = datetime.utcnow()
    # from influxdb.line_protocol._convert_timestamp
    nanos = int(timegm(timestamp.utctimetuple()) * 1e9 + timestamp.microsecond * 1e3)
    return nanos


def math_func(name, x, amplitude=10):
    """
    https://stackoverflow.com/questions/1073606/is-there-a-one-line-function-that-generates-a-triangle-wave

    .. see also::

        - https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.signal.sawtooth.html
        - https://gist.github.com/endolith/407991
        - https://code.activestate.com/recipes/577592-simple-1khz-audio-function-generator-using-standar/
    """
    x = float(x)
    y = x

    # triangle
    if name == "triangle":
        y = abs((x % (amplitude * 2)) - amplitude)
        # y = amplitude - abs(x % (2*amplitude) - amplitude)

    # square
    elif name == "square":
        y = (x % (amplitude * 2)) < amplitude and amplitude or 0

    # sawtooth
    elif name == "sawtooth":
        y = round((x / amplitude - int(x / amplitude)) * amplitude)

    # sine
    elif name == "sine":
        y = amplitude * sin(x / 10)

    else:
        raise NotImplementedError('Math func "{}" not implemented'.format(name))

    return y


def traceback_get_exception(num=-1):

    # build error message
    exception_string = "".join(
        traceback.format_exception_only(sys.exc_type, hasattr(sys, "exc_value") and sys.exc_value or "Unknown")
    )

    # extract error location from traceback
    if hasattr(sys, "exc_traceback"):
        (filename, line_number, function_name, text) = traceback.extract_tb(sys.exc_traceback)[num]
    else:
        (filename, line_number, function_name, text) = ("-", "-", "-", "-")

    error = {
        "message": exception_string,
        "location": {
            "filename": filename,
            "line_number": line_number,
            "function_name": function_name,
            "text": text,
        },
    }

    return error


def format_exception_location(error, prefix=""):
    if prefix:
        prefix += "\n"
    error_location = prefix + "Filename:    %s\nLine number: %s\nFunction:    %s\nCode:        %s" % (
        error["location"]["filename"],
        error["location"]["line_number"],
        error["location"]["function_name"],
        error["location"]["text"],
    )
    return error_location


def last_error_and_traceback():
    error_entry = traceback_get_exception(0)
    error_last = traceback_get_exception(-1)
    sep = "-" * 60
    payload_location = "\n".join(
        [
            sep,
            format_exception_location(error_entry, "Entry point:"),
            sep,
            format_exception_location(error_last, "Source of exception:"),
        ]
    )
    payload = "".join(["ERROR: ", error_entry["message"], "\n", payload_location, "\n"])

    # add full traceback
    buffer = StringIO()
    traceback.print_exc(file=buffer)
    buffer.seek(0)
    payload += "\n" + buffer.read()

    return payload


def get_hostname():
    return socket.gethostname()



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import json
import logging
import random
import sys
import time

from beradio.decoder import jobee_decode

try:
    from pprint import pprint
except ImportError:
    from pprint import pformat

    def pprint(x):
        print(pformat(x))


from .message import BERadioMessage
from .mqtt import BERadioMQTTAdapter
from .network import protocol_factory
from .util import math_func

"""
Read data in Bencode format from command line, decode and publish via MQTT.

Synopsis::

  # send given Bencode payload
  python publish.py 192.168.59.103 li999ei99ei1ei2218ei2318ei2462ei2250ee

  # send random values
  python publish.py 192.168.59.103 random

"""

logger = logging.getLogger(__name__)


class DataToMQTT(object):
    def __init__(self, mqtt_broker, mqtt_topic=None, protocol=2):
        self.mqtt_broker = mqtt_broker
        self.mqtt_topic = mqtt_topic
        self.protocol_class = protocol_factory(protocol)

    def setup(self):
        try:
            logger.info('Connecting to MQTT broker "{}"'.format(self.mqtt_broker))
            self.mqtt = BERadioMQTTAdapter(self.mqtt_broker, topic=self.mqtt_topic, enable_heartbeat=True)
        except:  # noqa:E722
            logger.error('Failed to connect to MQTT broker "{}"'.format(self.mqtt_broker))
            raise

        return self

    def publish(self, payload):

        logger.info('Setting up publishing of "{}"'.format(payload))

        # use mqtt.loop_forever() in a separate thread, this will handle
        # automatic reconnecting if connection to broker got lost
        self.mqtt.mqttc.loop_start()

        if payload == "random":
            if self.protocol_class.VERSION == 1:
                data = [
                    "999",
                    "99",
                    "2",
                    scale_100(random_temp()),
                    scale_100(random_temp()),
                    scale_100(random_temp()),
                    scale_100(random_temp()),
                ]
            elif self.protocol_class.VERSION == 2:
                data = {
                    "#": "999",
                    "_": "h1",
                    "t": [
                        scale_100(random_temp()),
                        scale_100(random_temp()),
                        scale_100(random_temp()),
                        scale_100(random_temp()),
                    ],
                    "h": [scale_100(random_hum()), scale_100(random_hum())],
                    "w": scale_100(random_weight()),
                }
            payload = self.protocol_class.encode_ether(data)
            logger.info("Publishing random payload: {}".format(payload))
            self.publish_real(payload)

        elif payload.startswith("func:"):

            func_name = payload.replace("func:", "")

            x = 1.0

            def apply_func(func_name, x, times):
                # shifting_factor = 3
                for i in range(times):
                    y = math_func(func_name, x + i * 3)
                    yield y

            # interval = 0
            # interval = 0.01
            interval = 0.5

            logger.info("Running publishing loop with interval={}".format(interval))
            while True:
                x += 1

                message = BERadioMessage(999, profile="h1")
                message.temperature(*apply_func(func_name, x, 4))
                message.humidity(*apply_func(func_name, x, 2))
                message.weight(math_func(func_name, x))
                # pprint(message)

                if logger.level != logging.DEBUG:
                    sys.stderr.write(".")

                # publish to MQTT
                self.publish_real(message.encode())

                time.sleep(interval)

        elif payload.startswith("json:"):
            json_payload = payload.replace("json:", "")
            data = json.loads(json_payload)

            node_id = data.get("node", "999")

            message = {
                "meta": {
                    "protocol": "mqttkit1",
                    "network": str(self.protocol_class.network_id),
                    "gateway": str(self.protocol_class.gateway_id),
                    "node": node_id,
                },
                "data": data,
            }

            self.mqtt.publish_json(message)

        elif payload.startswith("value:"):
            json_payload = payload.replace("value:", "")
            data = json.loads(json_payload)

            first = data.items()[0]
            name = first[0]
            value = first[1]

            # print name, value

            message = {
                "meta": {
                    "protocol": "mqttkit1",
                    "network": str(self.protocol_class.network_id),
                    "gateway": str(self.protocol_class.gateway_id),
                    "node": 999,
                },
                "data": data,
            }

            self.mqtt.publish_value(message, name, value)

        # Decode JobeeMonitor line format
        # id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;
        elif payload.startswith("id=") and ";" in payload:
            data = jobee_decode(payload)
            nodeid = data["id"]
            del data["id"]

            # Build an appropriate message from Jobee data
            message = {
                "meta": {
                    "network": str(self.protocol_class.network_id),
                    "gateway": str(self.protocol_class.gateway_id),
                    "node": nodeid,
                },
                "data": data,
            }
            self.mqtt.publish_flexible(message)

        else:
            self.publish_real(payload)

        self.mqtt.mqttc.loop_stop()

    def publish_real(self, payload):

        # decode wire format
        data = self.protocol_class.decode(payload)
        logger.debug("data: {}".format(data))

        if self.protocol_class.VERSION == 1:
            data = self.protocol_class.to_v2(data)

        # publish to MQTT
        self.mqtt.publish_flexible(data, bencode_raw=payload)

    def __del__(self):
        if hasattr(self, "mqtt"):
            logger.info("Disconnecting from MQTT broker")
            self.mqtt.close()


def random_temp():
    return round(random.uniform(7, 26), 2)


def random_hum():
    return round(random.uniform(3, 11), 2)


def random_weight():
    return round(random.uniform(100, 130), 2)


def scale_100(temp):
    return int(temp * 100)


if __name__ == "__main__":
    DataToMQTT(mqtt_broker=sys.argv[1], protocol=1).setup().publish(sys.argv[2])



import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, "README.rst")).read()
# CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    "pyserial<4",
    "paho-mqtt<2",
    "bencode.py<5",
    "docopt<1",
    "appdirs<2",
    "json-store<4",
    # 'terkin==0.0.1',
]

test_requires = [
    "nose2<1",
    "nose2-cov==1.0a4",
    "jsonpointer<3",
    "PyYAML<7",
    "u-msgpack-python<3",
]

setup(
    name="beradio",
    version="0.13.0",
    description="BERadio is an encoding specification and implementation for efficient "
    "communication in constrained radio link environments. It is conceived "
    "and used for over-the-air communication within the Hiveeyes project.",
    long_description=README,
    license="AGPL 3, EUPL 1.2",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Communications",
        "Topic :: Internet",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Archiving",
        "Topic :: Terminals :: Serial",
        "Topic :: Text Processing",
        "Topic :: Utilities",
    ],
    author="The Hiveeyes Developers",
    author_email="hello@hiveeyes.org",
    url="https://hiveeyes.org/docs/beradio/",
    keywords="protocol rfm serial mqtt Bencode BERadio",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite="beradio",
    install_requires=requires,
    tests_require=test_requires,
    extras_require={
        "test": test_requires,
    },
    entry_points={
        "console_scripts": [
            "beradio            = beradio.commands:beradio_cmd",
            "bdecode            = beradio.commands:bdecode_cmd",
            "bencode            = beradio.commands:bencode_cmd",
            "bemqtt             = beradio.commands:bemqtt_cmd",
        ],
    },
)



# -*- coding: utf-8 -*-
# (c) 2011 Eran Sandler <eran@sandler.co.il>
# (c) 2012 David Koblas
# (c) 2014 Học Đỗ <hoc3010@gmail.com>
# (c) 2015 Andreas Motl <andreas@hiveeyes.org>
# https://github.com/erans/pysnowflake
# https://github.com/koblas/pysnowflake
# https://github.com/tarzanjw/pysnowflake
# https://github.com/tarzanjw/pysnowflake/blob/master/snowflake/server/generator.py
import logging
import time

EPOCH_TIMESTAMP = 550281600000


class SnowflakeGenerator(object):
    def __init__(self, datacenter_id, worker_id):
        self.dc = datacenter_id
        self.worker = worker_id
        self.node_id = ((self.dc & 0x03) << 8) | (self.worker & 0xFF)
        self.last_timestamp = EPOCH_TIMESTAMP
        self.sequence = 0
        self.sequence_overload = 0
        self.errors = 0
        self.generated_ids = 0

    def get_next_id(self):
        curr_time = int(time.time() * 1000)

        if curr_time < self.last_timestamp:
            # stop handling requests til we've caught back up
            self.errors += 1
            raise AssertionError("Clock went backwards! %d < %d" % (curr_time, self.last_timestamp))

        if curr_time > self.last_timestamp:
            self.sequence = 0
            self.last_timestamp = curr_time

        self.sequence += 1

        if self.sequence > 4095:
            # the sequence is overload, just wait to next sequence
            logging.warning("The sequence has been overload")
            self.sequence_overload += 1
            time.sleep(0.001)
            return self.get_next_id()

        generated_id = ((curr_time - EPOCH_TIMESTAMP) << 22) | (self.node_id << 12) | self.sequence

        self.generated_ids += 1
        return generated_id

    @property
    def stats(self):
        return {
            "dc": self.dc,
            "worker": self.worker,
            "timestamp": int(time.time() * 1000),  # current timestamp for this worker
            "last_timestamp": self.last_timestamp,  # the last timestamp that generated ID on
            "sequence": self.sequence,  # the sequence number for last timestamp
            "sequence_overload": self.sequence_overload,  # the number of times that the sequence is overflow
            "errors": self.errors,  # the number of times that clock went backward
        }
